/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.MainApplication");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.rational.report.web.core.app.internal.ReportApplication");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ProvisioningClient");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");

var CommonMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var InternalMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var ReportApplication		= com.ibm.rational.report.web.core.app.internal.ReportApplication;
var ProvisioningClient		= com.ibm.rational.report.web.core.client.internal.ProvisioningClient;
var InitializationData      = com.ibm.team.repository.web.client.init.InitializationData;
var isGuest					= com.ibm.team.repository.web.client.session.isGuest;
var PageContainer			= net.jazz.ajax.ui.PageContainer;
var RepositoryClient 		= com.ibm.team.repository.web.client.internal.RepositoryClient;
var ProvisionResponseHandler = com.ibm.team.repository.web.transport.internal.ProvisionResponseHandler;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util 					= com.ibm.rational.report.web.core.app.internal.Util;

function getProjectAreaName() {
	return com.ibm.team.process.web.projectArea.name;
}

function endsWith(str, end) {
	if((str.length - end.length) < 0) {
		return false;
	}
	return str.lastIndexOf(end) == (str.length - end.length);
}

/**
 * Displays the Reports perspective and artifacts associated with a project area.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.MainApplication",
	ReportApplication, {
	
	init: function() {
		var initialLoadMsg = dojo.byId("net-jazz-ajax-InitialLoadMessage");
		Util.removeChildren(initialLoadMsg);
		dojo.addClass(initialLoadMsg, "displayLoadingMsg");
		initialLoadMsg.appendChild(document.createTextNode(InternalMessages.LOADING_PROGRESS_MSG));
		var prh = new ProvisionResponseHandler(this, "_authRequestPassed", "_authRequestFailed");
		ProvisioningClient.isRepositoryAvailable(prh);
	},
	
	_authRequestPassed: function () {
		this._fetchInitData();
	},
	
	_authRequestFailed: function(errorObject) {
		this._fetchInitDataFailed(errorObject);
	},
	
	_fetchInitData: function () {
		var srh = new ServiceResponseHandler(this, "_handleInitData", "_fetchInitDataFailed");
		RepositoryClient.getInitializationData(srh);
	},
	
	_handleInitData: function(initData) {
		InitializationData._load(initData);
		
		// handle case where there is no DB so contributor isn't in init data
		if (InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer") != null) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR",
			InitializationData.get("com.ibm.team.repository.service.currentContributorInitializer").currentContributor);
		}
				
		// set the global far for the current project area based on the last
		// segment of the document's URI and the set of project areas in the
		// repository
		var pathname = window.location.pathname;
		if (endsWith(pathname, "/")) {
			pathname = pathname.substring(0, pathname.length - 1);
		}
		var projectName = pathname.substring(pathname.lastIndexOf('/') + 1);
		var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas;	
		var projectArea = dojo.setObject("com.ibm.team.process.web.projectArea", {});
		for (var i = 0; i < projectAreas.length; i++) {
			if (projectAreas[i].name == unescape(projectName)) {
				projectArea.itemId = projectAreas[i].itemId;
				projectArea.name = projectAreas[i].name;
				break;
			}	
		}
		
		if (!projectArea.itemId) {
			var validProjectArea = this.handleInvalidProjectArea(pathname, projectAreas);
			if (typeof validProjectArea != "undefined" && validProjectArea != null) {
				projectArea.name = validProjectArea.name;
				projectArea.itemId = validProjectArea.itemId;
				window.location = pathname + "/projects/" + projectArea.name;
			}
		}
		
		if (!projectArea.itemId) {
			this._fetchProjectAreaFailed();
		}
		else {
			// store the configuration properties in global vars with corresponding names
			var configProperties = InitializationData.get("com.ibm.team.repository.service.configPropertyInitializer");
			for (var propName in configProperties) {
				var propVal = configProperties[propName];
				dojo.setObject(propName, propVal);
			}
			
			this._startApp();
		}
	},
	
	_startApp: function(){		
		var args = {
			label: "Reportal Web UI",
			showLoginPanel: true
		};
		this._initUI(args);
	
		MainApplication.superclass.start.call(this);
	},
	
	_initUI: function() {
		this.inherited(arguments);
	},
	
	_fetchInitDataFailed: function() {
		var self = this;
		dojo.addClass(dojo.body(), "com-ibm-rational-report-web com-ibm-team-repository-web");		
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.ERROR, prompt: ServiceabilityMessages.NOT_GRANTED_WEBSITE_PERMISSION, promptType: "Error", buttonType: "Ok"});
		dojo.connect(errorDialog.rightButton, "onclick", errorDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				errorDialog.close();
				self._exitUI();
			}
		);
	},
	
	_fetchProjectAreaFailed: function(errorObject) {
		var self = this;
		dojo.addClass(dojo.body(), "com-ibm-rational-report-web com-ibm-team-repository-web");
		var reason = ServiceabilityMessages.FAILED_TO_RETRIEVE_PROJECTS;
		var index = reason.indexOf(":");
		if (index > 0) {
			reason = reason.substring(index + 2);
		}
		var msg = dojo.string.substitute(
			ServiceabilityMessages.INTERNAL_APPLICATION_ERROR_WITH_REASON, [reason]);
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.ERROR, prompt: msg, promptType: "Error", buttonType: "Ok"});
		dojo.connect(errorDialog.rightButton, "onclick", errorDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				errorDialog.close();
				self._exitUI();
			}
		);
	},	
	
	_exitUI: function() {
		var bindArgs = {
			url: net.jazz.ajax._contextRoot + "/auth/logout",
			handleAs: "json",
			headers: {
				accept: "text/json"
			},
			preventCache: dojo.isIE,
			handle: function(response, ioArgs) {
				document.location.reload();
			}
		};
		dojo.xhrPost(bindArgs);
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	},
	
	handleInvalidProjectArea: function(pathname, projectAreas) {	
	}
	
});

var MainApplication = com.ibm.rational.report.web.core.app.internal.MainApplication;

})();
