/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.ReportContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportParameterViewer");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportParameterManager");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Displays the rendered reports in the main content area.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.ReportContent",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/ReportContent.html"),
	
	// Arguments:
	//     name - mandatory name of the report
	//     parameterName - optional array of parameter names
	//     parameterValue - optional array of parameter values
	//     collapseParameters - boolean
	constructor: function(args) {
		this._controller = args.controller;
		this._reportItemId = args.reportItemId;
		this._parentWidget = args.parentWidget;
		this.messageArea = this._parentWidget.messageArea;
		this._args = args;
	},
	
	postCreate: function() {
		this._msgFormatter = new MessageFormatter(this.messageArea);		
		var self = this;		
		var args = this._args;
		
		this._teamAreas = [];		
		this._projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		this._parseTeamAreas(this._projectArea.children);
		
		var requestParams = {
			id: args.reportItemId
		};
		
		this._msgFormatter.displayLoadingMessage(); 
		
		var srh = new ServiceResponseHandler(this, "_displayTitle",	"_remoteError");
		if(args.type == "REPORT_DESIGN"){
			ReportClient.getReportDesign(srh, requestParams);
		} else if(args.type == "REPORT_QUERY"){
			ReportClient.getReportQuery(srh, requestParams);
		} else if(args.type == "REPORT_SNAPSHOT"){
			ReportClient.getReportSnapshot(srh, requestParams);
		}
	},
	
	_displayTitle: function(report) {
		this._report = report;			
		var self = this;
		var args = this._args;		
		
		var args = {id: this._args.reportItemId};
		if (this._args.design != null) {
			args.design = this._args.design;
			this._design = args.design;
		}
		var srh = new ServiceResponseHandler(this, "_displayReportParameters",	"_remoteError");
		ReportClient.getReportParameterDefs(srh, args);
	},
	
	_displayReportParameters: function(parameterDefs){
		this._msgFormatter.displayNoMessage();
		var design = null;
		if (this._args.design != null) {
			design = this._args.design;
		}
		this._parameterManager = new com.ibm.rational.report.web.core.app.view.report.ReportParameterManager(
			this._report, design, parameterDefs, this._args.reportParamNames, this._args.reportParamValues);
		if(this._parameterManager.hasVisibleParams()){
			this._parameterViewer = new com.ibm.rational.report.web.core.app.view.report.ReportParameterViewer({
				paramManager: this._parameterManager});
			this.parametersDiv.appendChild(this._parameterViewer.domNode);
		}
		if(this._parameterManager.hasAllRequiredValues()){
			this._getRenderedReport();
		}
		var self = this;
		dojo.subscribe("com.ibm.rational.report.runReport."+this._parameterManager.report.itemId, 
			function() {
				self._getRenderedReport();
			}
		);
	},
	
	_getRenderedReport: function() {
		this._msgFormatter.displayLoadingMessage(); 
		var args = {id: this._args.reportItemId};
		if (this._args.design != null) {
			args.design = this._args.design;
			this._design = args.design;
		}
		var parameters = this._parameterManager.getParamNameAndValues();
		args.reportParamNames = parameters.names;
		args.reportParamValues = parameters.values;
		var srh = new ServiceResponseHandler(this, "_displayRenderedReport",	"_remoteError");
		ReportClient.getRenderedReport(srh, args);
	},
	
	_parseTeamAreas: function(teamAreas) {	
		if (teamAreas == null || teamAreas.length == 0) {
			return;
		}
		for (var i = 0; i < teamAreas.length; i++) {
			var teamArea = teamAreas[i];
			this._teamAreas.push(teamArea.name);		
			this._parseTeamAreas(teamArea.children);
		}
	},
	
			
	_done: function(result) {
		this._msgFormatter.displayNoMessage();
		if (this._postServiceExec != null) {
			this._postServiceExec();
		}
	},
	
	_displayRenderedReport: function(result) {
//		if (this._gotParametersResponse) {
			// Now have both responses, remove the loading spinner
			this._msgFormatter.displayNoMessage();
//		}
//		this._gotRenderedResponse = true;
		if (navigator.appVersion.indexOf("MSIE")!=-1) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this.renderedReportDiv.innerHTML = result;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			var newNode = document.createElement("div");
			newNode.innerHTML = result;
			Util.removeChildren(this.renderedReportDiv);
			this.renderedReportDiv.appendChild(newNode);
		} 
	},

	_displayParametersSection: function(result) {
		if (this._gotRenderedResponse) {
			// Now have both responses, remove the loading spinner
			this._msgFormatter.displayNoMessage();
		}
		this._gotParametersResponse = true;
		if (navigator.appVersion.indexOf("MSIE")!=-1) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this.parametersDiv.innerHTML = result;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			var newNode = document.createElement("div");
			newNode.innerHTML = result;
			Util.removeChildren(this.parametersDiv);
			this.parametersDiv.appendChild(newNode);
		}
		
		var srh = new ServiceResponseHandler(this, "_displayRenderedReport", "_remoteError");
		ReportClient.getRenderedReport(srh, this._clientArgs);
	},
	
	_postActionFunc: function(){
		this._controller.applySelectedTeamAreas();
	},

	_remoteError: function(error) {
		this._msgFormatter.displayNoMessage();
		var msg = this._msgFormatter.formatErrorMessage(ServiceabilityMessages.FAILED_TO_RENDER_REPORT, error);
		this._displayErrorMessage(msg);
		this._controller.clearMainContent();
	},
	
	_sendError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_SEND_REPORT_EMAIL, error);
	},
	
	_isArray: function(obj) {
	   if (obj.constructor.toString().indexOf("Array") == -1)
	      return false;
	   else
	      return true;
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}
	
});

})();